import React, {Component, useEffect, useState} from 'react';
import './App.css';
import {FwoType} from "./models/fwo";
import {MwoType} from "./models/mwo";
import {MwoLineType} from "./models/mwoLineType";
import {WksType} from "./models/wks";
import {StockType} from "./models/stock";
import {MaterialType} from "./models/material";
import {DataDynamics} from "./ui/DataDynamics";
import {TabBody} from "./ui/body";
import Header from "./ui/header";

type IdCalendar = string
type intervalType = {
    name: string
    cls?: string
    endDate?: Date
    iconCls?: string
    isWorking: boolean
    recurrentEndDate?: string
    recurrentStartDate?: string
    startDate?: Date
}

type CalendarType = {
    id: IdCalendar
    name: string
    unspecifiedTimeIsWorking: boolean
    cls?: string
    intervals: intervalType[]
}

type CoreType = {
    fwos: FwoType[],
    mwos: MwoType[],
    mwoLines: MwoLineType[],
    wks: WksType[],
    calendars: CalendarType[],
    stocks: StockType[],
    material: MaterialType[],
}


type SwitchTabType = (tab: MainTab) => void

type setFwosType = React.Dispatch<React.SetStateAction<FwoType[]>>
type setMwosType = React.Dispatch<React.SetStateAction<MwoType[]>>
type setMwoLinesType = React.Dispatch<React.SetStateAction<MwoLineType[]>>
type setWksType = React.Dispatch<React.SetStateAction<WksType[]>>
type setCalendarsType = React.Dispatch<React.SetStateAction<CalendarType[]>>
type setStocksType = React.Dispatch<React.SetStateAction<StockType[]>>
type setMaterialType = React.Dispatch<React.SetStateAction<MaterialType[]>>

type setPlanStateType = React.Dispatch<React.SetStateAction<autoPlanStateType>>
enum autoPlanStateEnum {
    //TODO add need check or something
    //dont have more than needed
    ok,
    failed,

    doLoad,
    sudoLoad,
    loading,

    doCheck,
    sudoCheck,
    checking,

    doPlan,
    doReplan,
    sudoPlan,
    sudoReplan,
    planning,

    doSave,
    sudoSave,
    saving,
    // justLoaded, //isnt needed right now
}


type autoPlanStateType = {
    type: autoPlanStateEnum //missing type and startDate indicate no need to autoplan
    startDate: Date
    afterDate?: Date
    beforeDate?: Date
}

enum MainTab {
    Timeline = 1,
    GridMwos,
    GridStocks,
}

function App() {
    const [fwos, setFwos] = useState<FwoType[]>([])
    const [mwos, setMwos] = useState<MwoType[]>([])
    const [mwoLines, setMwoLines] = useState<MwoLineType[]>([])
    const [wks, setWks] = useState<WksType[]>([])
    const [calendars, setCalendars] = useState<CalendarType[]>([])
    const [stocks, setStocks] = useState<StockType[]>([])
    const [material, setMaterial] = useState<MaterialType[]>([])

    const [activeMainTab, setActiveMainTab] = useState(MainTab.Timeline)
    const [planState, setPlanState] = useState<autoPlanStateType>({
        type: autoPlanStateEnum.loading,
        startDate: new Date()
    })

    useEffect(() => {
        //const element = document.getElementById('schedule_container')!
        //element.style.width = '80%'

        if (mwos.length === 0) {
            setPlanState(prevState => {
                return {...prevState, type: autoPlanStateEnum.sudoLoad}
            })
        }


    }, [mwos])

    return (
        <div id={'app'}>
            <DataDynamics
                setFwos={setFwos}
                setMwos={setMwos}
                setMwoLines={setMwoLines}
                setWks={setWks}
                setCalendars={setCalendars}
                setStocks={setStocks}
                setMaterial={setMaterial}

                fwos={fwos}
                mwos={mwos}
                mwoLines={mwoLines}
                wks={wks}
                calendars={calendars}
                stocks={stocks}
                material={material}
                planState={planState}
                setPlanState={setPlanState}
                />
            <Header switchTab={setActiveMainTab}></Header>
            <TabBody
                fwos={fwos}
                setFwos={setFwos}
                mwos={mwos}
                setMwos={setMwos}
                mwoLines={mwoLines}
                wks={wks}
                calendars={calendars}
                stocks={stocks}
                material={material}
                activeMainTab={activeMainTab}
                planState={planState}
                setPlanState={setPlanState}
            />
        </div>
    )
}

export default App;

export {autoPlanStateEnum, MainTab};

export type {
    CoreType,
    SwitchTabType,
    setFwosType,
    setMwosType,
    setMwoLinesType,
    setWksType,
    setCalendarsType,
    setStocksType,
    autoPlanStateType,
    setMaterialType,
    setPlanStateType
}