import {FwoType} from "../models/fwo";
import {CoreType} from "../App";
import {MwoType} from "../models/mwo";
import {API} from "../ui/DataDynamics";
import {data} from "../data/data";


function ISOtoDate(s: string): Date {
    if (!s.endsWith('Z')) {
        s += 'Z'
    }
    return new Date(s)
}

function ISOtoDateMaybe(s: string | null): Date | null {
    if (s !== null && s !== undefined) {
        return ISOtoDate(s)
    }
    return null
}

function fixMwo(mwo: any): MwoType {
    return {
        ...mwo,
        dueDate: ISOtoDate(mwo.dueDate),
        startPlanned: ISOtoDateMaybe(mwo.startPlanned),
        startActual: ISOtoDateMaybe(mwo.startActual),
        endActual: ISOtoDateMaybe(mwo.endActual),
        modifyingDate: ISOtoDateMaybe(mwo.modifyingDate),
        //predecessors: mwo.predecessors,
        startPlannedLocked: mwo.startPlannedLocked ?? false,
        errors: mwo.errors ?? []
    }
}

function fixMwos(mwos: any[]): MwoType[] {
    return mwos.map(fixMwo)
}


function fixFwo(fwo: any): FwoType {
    return {
        ...fwo,
        dueDate: ISOtoDateMaybe(fwo.dueDate),
        startPlanned: ISOtoDateMaybe(fwo.startPlanned),
    }
}

function fixFwos(fwos: any): FwoType[] {
    return fwos.map(fixFwo)
}
// Only fix the ones there is no workaround like dates.
// Everything elese must be fixed by backend

function fixJson(json: any): Promise<CoreType> {
    return {...json, mwos: fixMwos(json.mwos), fwos: fixFwos(json.fwos)}
}
class Api implements API {
    async loadData(): Promise<CoreType | undefined> {
        return fixJson(data)
    }
}

export {Api}