import {
    autoPlanStateEnum,
    autoPlanStateType,
    CoreType,
    setCalendarsType,
    setFwosType,
    setMaterialType,
    setMwoLinesType,
    setMwosType, setPlanStateType, setStocksType,
    setWksType
} from "../App";
import {WksType} from "../models/wks";
import {MwoType} from "../models/mwo";
import {MaterialType} from "../models/material";
import {FwoType} from "../models/fwo";
import {StockType} from "../models/stock";
import {MwoLineType} from "../models/mwoLineType";
import {useEffect} from "react";
import {Api} from "../api/api";
import {CalendarType} from "./timeline/timeline";

interface API {
    loadData(): Promise<CoreType | undefined>

}


type DataSyncPropsType = {
    setFwos: setFwosType
    setMwos: setMwosType
    setMwoLines: setMwoLinesType
    setWks: setWksType
    setCalendars: setCalendarsType
    setStocks: setStocksType
    setMaterial: setMaterialType

    fwos: FwoType[],
    mwos: MwoType[],
    mwoLines: MwoLineType[],
    wks: WksType[],
    //calendars: CalendarType[],
    stocks: StockType[],
    material: MaterialType[],

    calendars: CalendarType[],

    planState: autoPlanStateType,

    setPlanState: setPlanStateType,
}

const DataDynamics = (props: DataSyncPropsType) => {
    const source: API = new Api()
    function updateLoadData(data: CoreType | undefined) {
        if (data) {
            props.setFwos(data.fwos)
            props.setMwos(data.mwos)
            props.setMwoLines(data.mwoLines)
            props.setWks(data.wks)
            props.setCalendars(data.calendars)
            props.setStocks(data.stocks)
            props.setMaterial(data.material)
        }
    }

    useEffect(() => {
        const ps = props.planState

        switch (ps.type) {
            case autoPlanStateEnum.sudoLoad: {
                props.setPlanState(prevState => {
                    return {...prevState, type: autoPlanStateEnum.loading}
                })
                source.loadData()
                    .then(updateLoadData)
                //Dont explicitly change planState here. Wait for props.mwo changes
                break
            }
        }

    }, [props.planState])


    return (
        <div></div>
    )
}

export {DataDynamics}
export type {API}