import {DateHelper, DragHelper, DomHelper, Rectangle, SchedulerPro} from '@bryntum/schedulerpro'

class Drag extends DragHelper {
    static get configurable() {
        return {
            callOnFunctions: true,
            // Don't drag the actual row element, clone it
            cloneTarget: true,
            // We size the cloned element manually
            autoSizeClonedTarget: false,
            // Only allow drops on the schedule area
            dropTargetSelector: '.b-timeline-subgrid',
            // Only allow drag of row elements inside on the unplanned grid
            targetSelector: '.b-grid-row:not(.b-group-row)'
        }
    }

    constructor(config: any) {
        super(config)
        const me: any = this

        // Configure DragHelper with schedule's scrollManager to allow scrolling while dragging
        me.scrollManager = config.scheduleInstance.scrollManager
        //me.schedule.mwos = config.mwos
        //me.schedule.useMwos = config.useMwos
        //me.scrollManager = me.schedule.scrollManager

        me.scheduleInstance = config.scheduleInstance
        me.histogramInstance = config.histogramInstance
        me.gridInstance = config.gridInstance
        me.addDraggedElement = config.addDraggedElement
        me.on({
            dragstart: me.onTaskDragStart,
            drag: me.onTaskDrag,
            drop: me.onTaskDrop,
            //mwos: me.mwos,
            //setMwos: me.useMwos,
            thisObj: me
        })
    }

    onTaskDragStart({context}: any) {
        const me: any = this,
            {schedule} = me,
            mouseX = context.clientX,
            proxy = context.element,
            task = me.grid.getRecordFromElement(context.grabbed),
            //task.endDate.getTime() - task.startDate.getTime()
            newWidth = me.scheduleInstance.timeAxisViewModel.getDistanceForDuration(task.duration * 1000)

        // save a reference to the task so we can access it later
        context.task = task

        // Mutate dragged element (grid row) into an event bar
        proxy.classList.remove('b-grid-row')
        proxy.classList.add('b-sch-event-wrap')
        proxy.classList.add('b-sch-event')
        proxy.classList.add('b-unassigned-class')
        proxy.classList.add(`b-${schedule.mode}`)


        proxy.innerHTML = `<i class="${task.iconCls}"></i> ${task.name}`

        // If the new width is narrower than the grabbed element...
        if (context.grabbed.offsetWidth > newWidth) {
            const proxyRect = Rectangle.from(context.grabbed)

            // If the mouse is off (nearly or) the end, centre the element on the mouse
            if (mouseX > proxyRect.x + newWidth - 20) {
                context.newX = context.elementStartX = context.elementX = mouseX - newWidth / 2
                DomHelper.setTranslateX(proxy, context.newX)
            }
        }

        proxy.style.width = `${newWidth}px`

        // Prevent tooltips from showing while dragging
        me.scheduleInstance.element.classList.add('b-dragging-event')
    }

    onTaskDrag({context}: any) {
        const me: any = this,
            {schedule} = me,

            {task} = context,
            coordinate = DomHelper[`getTranslate${schedule.isHorizontal ? 'X' : 'Y'}`](
                context.element
            ),
            startDate = schedule.getDateFromCoordinate(coordinate, 'round', false),
            endDate = startDate && DateHelper.add(startDate, task.duration, task.durationUnit),
            // Coordinates required when used in vertical mode, since it does not use actual columns
            resource = context.target && schedule.resolveResourceRecord(context.target)

        // Don't allow drops anywhere, only allow drops if the drop is on the timeaxis and on top of a Resource
        context.valid &=
            Boolean(startDate && resource) &&
            (schedule.allowOverlap ||
                schedule.isDateRangeAvailable(startDate, endDate, null, resource))

        // Save reference to resource so we can use it in onTaskDrop
        context.resource = resource
    }

    // Drop callback after a mouse up, take action and transfer the unplanned task to the real SchedulerEventStore (if it's valid)
    onTaskDrop({context}: any) {
        const me: any = this,
            task = context.task,
            target = context.target

       me.scheduleInstance.element.classList.remove('b-dragging-event')

        if (context.valid && target) {
            const date = me.scheduleInstance.getDateFromCoordinate(
                DomHelper.getTranslateX(context.element),
                'round',
                false,
            )

            if (date) {

                const mwoInsert = {
                    id: task.id,
                    startDate: date,
                    durationUnit: 'second',
                    resizable: false,
                }


                me.addDraggedElement(mwoInsert)
            }
        }

    }
}

export {Drag}